/**
   AudioChain Copyright (C) 2012-2013 Florian Paul (www.audiochain.org)
   
   This program is free software; you can redistribute it and/or modify it under the terms of the 
   GNU General Public License as published by the Free Software Foundation; either version 3 of the 
   License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
   Public License for more details.
   
   You should have received a copy of the GNU General Public License along with this program; 
   if not, see <http://www.gnu.org/licenses/>. 
 */
package org.audiochain.devices.vst;

import java.io.IOException;

import org.audiochain.io.AbstractAudioDataReaderChainLink;
import org.audiochain.io.AudioDataReader;

import com.synthbot.audioplugin.vst.vst2.JVstHost2;

public class VstAudioDataReaderChainLink extends AbstractAudioDataReaderChainLink {
	private final JVstHost2 vstHost;
	private final boolean replacing;
	private float[][] input;
	private float[][] output;

	public VstAudioDataReaderChainLink(AudioDataReader source, JVstHost2 vstHost) {
		super(source);
		if (vstHost == null) {
			throw new IllegalArgumentException("vstHost must not be null");
		}
		this.vstHost = vstHost;
		replacing = vstHost.canReplacing();
	}

	@Override
	public int read(int[] data) throws IOException {
		int len = source.read(data);
		if (input == null || output == null) {
			int length = data.length / 2;
			int numInputs = vstHost.numInputs();
			input = new float[numInputs][length];
			int numOutputs = vstHost.numOutputs();
			output = new float[numOutputs][length];
		}
		int halfLen = len / 2;
		copy(data, input, halfLen);
		if (replacing) {
			vstHost.processReplacing(input, output, halfLen);
		} else {
			vstHost.process(input, output, halfLen);
		}
		copy(output, data, halfLen);
		return len;
	}

	private static void copy(int[] from, float[][] to, int len) {
		int channels = to.length;
		switch (channels) {
		default:
			// case of three or more channels
		case 2:
			copy(from, 1, to[1], len);
		case 1:
			copy(from, 0, to[0], len);
		case 0:
			return;
		}
	}

	private static void copy(int[] from, int startIndex, float[] to, int len) {
		for (int i = 0; i < len; i++) {
			int f = from[i * 2 + startIndex];
			float t = f; // TODO convert to float sample value
			to[i] = t;
		}
	}

	private static void copy(float[][] from, int[] to, int len) {
		int channels = from.length;
		switch (channels) {
		default:
			// case of three or more channels
		case 2:
			copy(from[1], to, 1, len);
		case 1:
			copy(from[0], to, 0, len);
		case 0:
			return;
		}
	}

	private static void copy(float[] from, int[] to, int startIndex, int len) {
		for (int i = 0; i < len; i++) {
			float f = from[i];
			int t = (int) f; // TODO convert to int sample value
			to[i * 2 + startIndex] = t;
		}
	}
}
